library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity PMC is port(
    CLK : in  std_logic;    -- system clock, 24MHz for 4Mbps
    SD  : inout std_logic;  -- serial data, high active
    FB  : out std_logic;    -- feedback of SD input
    OE  : out std_logic;    -- output enable of SD
    STP,DIR  : out std_logic;  -- motor step pulse, direction
    M1,M2,M3 : out std_logic;  -- motor step angle
    PWR : out std_logic;    -- motor power control
    P   : out std_logic_vector(3 downto 0);   -- PWR data
    SYN : out std_logic;    -- motor syncronized chopping
    SE  : in  std_logic_vector(3 downto 0));  -- sensor signals
end PMC;

architecture RTL of PMC is
signal Q0,Q1 : std_logic := '0'; -- edge detection of SD
signal DEM   : std_logic_vector(2 downto 0) := "000"; -- demodulator
signal SCK   : std_logic := '0'; -- detected serial clock
signal SCK0  : std_logic := '0'; -- center of bit cell
signal SCK1  : std_logic := '0'; -- for SCI
signal SCK3  : std_logic := '0'; -- bit boundary
signal R0,R1 : std_logic := '0'; -- registerd Q0 by SCK
signal RSD   : std_logic := '0'; -- demodulated recieved serial data
signal SCI   : std_logic_vector(3 downto 0) := "0000";
signal RDR   : std_logic_vector(8 downto 0) := "000000000";
signal M3_fb : std_logic := '0'; -- internal feedback of M3
signal P_fb  : std_logic_vector(3 downto 0) := "0000";
signal PWM   : std_logic_vector(4 downto 0) := "00000";
signal CMR   : std_logic_vector(4 downto 0) := "00000";
signal TDR   : std_logic_vector(3 downto 0) := "0000";
signal TSD   : std_logic := '0'; -- transmit serial data
signal SDO   : std_logic := '0'; -- modulated transmit serial data
signal OE_fb : std_logic := '0'; -- SD output enable

begin
-- SCK generator ----------------------------------------------
process (CLK) begin
    if (CLK'event and CLK = '1') then
        Q0 <= SD;
        Q1 <= Q0;
    end if;
end process;
process (SCI, DEM, Q0, Q1) begin
    if (SCI <= "1001") or (SCI = "1111") then  -- recieve bits
        if (DEM >= "100") and (Q0 /= Q1) then  -- bit boundary
            SCK <= '1';
        else
            SCK <= '0';
        end if;
    else  -- (SCI >= "1010") and (SCI <= "1110")  transmit bits
        if (DEM >= "101") then  -- bit boundary
            SCK <= '1';
        else
            SCK <= '0';
        end if;
    end if;
end process;
    SCK0 <= '1' when (DEM = "000") else '0';  -- center of bit cell
    SCK1 <= '1' when (DEM = "001") else '0';  -- for SCI
    SCK3 <= '1' when (DEM = "011") else '0';  -- bit boundary

-- FSK demodulator --------------------------------------------
process (CLK) begin
    if (CLK'event and CLK = '1') then
        if (SCK = '1') then
            DEM <= "000";
            R0  <= Q0;
            R1  <= R0;
        elsif (DEM = "111") then
            DEM <= "111";
            R0  <= R0;
            R1  <= R1;
        else
            DEM <= DEM + '1';
            R0  <= R0;
            R1  <= R1;
        end if;
    end if;
end process;
    RSD <= R0 xnor R1;

-- Serial Communication Interface controler -------------------
process (CLK) begin
    if (CLK'event and CLK = '1') then
        if (SCK1 = '1') then
            if (SCI = "0000") then   -- idle state
                if (RSD = '1') then  -- start bit
                    SCI <= "0001";   -- recieve start
                else  -- stop bit
                    SCI <= "0000";   -- idle state
                end if;
            else  -- (SCI /= "0000"), not idel state
                    SCI <= SCI + '1';
            end if;
        else  -- (SCK1 = '0')
                    SCI <= SCI;
        end if;
    end if;
end process;

-- recieve data register --------------------------------------
process (CLK) begin
    if (CLK'event and CLK = '1') then
        if (SCK = '1') then
            case SCI is
                when "0010" =>  RDR(0) <= RSD;  -- M1
                when "0011" =>  RDR(1) <= RSD;  -- M2
                when "0100" =>  RDR(2) <= RSD;  -- M3
                when "0101" =>  RDR(3) <= RSD;  -- DIR
                when "0110" =>  RDR(4) <= RSD;  -- PWR(0)
                when "0111" =>  RDR(5) <= RSD;  -- PWR(1)
                when "1000" =>  RDR(6) <= RSD;  -- PWR(2)
                when "1001" =>  RDR(7) <= RSD;  -- PWR(3)
                when "1010" =>  RDR(8) <= RSD;  -- step pulse
                when others =>  RDR    <= RDR;
            end case;
        else
            RDR <= RDR;
        end if;
    end if;
end process;

-- motor control signals --------------------------------------
process (CLK) begin
    if (CLK'event and CLK = '1') then
        if (SCK = '1') and (SCI = "1011") then
            M1    <= RDR(0);  -- motor step angle setting
            M2    <= RDR(1);  -- motor step angle setting
            M3_fb <= RDR(2);  -- motor step angle setting
            DIR   <= RDR(3);  -- direction of motor rotation
            P_fb  <= RDR(7 downto 4);
            STP   <= RDR(8);  -- motor step pulse, rising edge active
        end if;
    end if;
end process;
    M3 <= M3_fb;
    P  <= P_fb;
    CMR <= "0" & P_fb;  -- compere match register
    SYN <= '1' when (M3_fb = '0') and (CMR = "00000") else '0';

-- pulse width modulator for motor power control --------------
process (CLK) begin
    if (CLK'event and CLK = '1') then
        PWM <= PWM + '1';
    end if;
end process;
    PWR <= '1' when (PWM <= CMR) or (PWM >= "11101") else '0';

-- Transmit Data Register -------------------------------------
process (CLK) begin
    if (CLK'event and CLK = '1') then
        if (SCK1 = '1') and (SCI = "1001") then
            TDR <= SE;
        else
            TDR <= TDR;
        end if;
    end if;
end process;

-- TSD serializer ---------------------------------------------
process (SCI) begin
    case SCI is
        when "1010" => TSD <= TDR(0);
        when "1011" => TSD <= TDR(1);
        when "1100" => TSD <= TDR(2);
        when "1101" => TSD <= TDR(3);
        when others => TSD <= '0';  -- STOP bit, dummy bit
    end case;
end process;

-- FSK modulator ----------------------------------------------
process (CLK) begin
    if (CLK'event and CLK = '1') then
        if (SCI = "1001") and (SCK = '1') then
                SDO <= Q0;            -- get initial level
        elsif (SCK0 = '1') then       -- center of bit cell
                SDO <= SDO xor TSD;   -- if TSD ='1' then invert
        elsif (SCK3 = '1') then       -- bit boundary
                SDO <= not SDO;
        else
                SDO <= SDO;
        end if;
    end if;
end process;

-- SD output enabel -------------------------------------------
process (CLK) begin
    if (CLK'event and CLK = '1') then
        if (SCI = "1001") and (DEM <= "001") then
            OE_fb <= '0';
        elsif (SCI >= "1010") and (SCI <= "1110") then
            OE_fb <= '0';
        else
            OE_fb <= '1';
        end if;
    end if;
end process;
    OE <= OE_fb;
    SD <= SDO when (OE_fb = '0') else 'Z';
    FB <= SD  when (OE_fb = '1') else 'Z';

end RTL;
