library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity RPC is port(
    RES   : in    std_logic;  -- system reset, low active
    MCK   : in    std_logic;  -- CPU clock
    DIN   : in    std_logic_vector(7 downto 0);  -- data bus
    DOUT  : out   std_logic_vector(3 downto 0);  -- data bus
    CS,RD,WR : in std_logic;  -- CPU interface
    STP   : in    std_logic;  -- motor step pulse
    CLK   : in    std_logic;  -- 24MHz for 4Mbps
    SD    : inout std_logic;  -- serial data
    FB    : out   std_logic;  -- feedback of SD input
    OE    : out   std_logic;  -- output enabel of SD
    SCK   : out   std_logic;  -- serial clock
    A,B   : in    std_logic;  -- rotary encoder pulse
    DIR   : inout std_logic;  -- direction of encoder
    MODE  : in    std_logic;  -- select motor step pulse input
    EDG   : inout std_logic); -- encoder pulse edge
end RPC;

architecture RTL of RPC is
component PRS port(           -- prescaler
    RES   : in    std_logic;  -- system reset, low active
    CLK   : in    std_logic;  -- 24MHz for 4Mbps
    SCKX  : inout std_logic_vector(2 downto 0));
end component;
signal SCKX : std_logic_vector(2 downto 0);

component SCI port(           -- serial communication interface
    RES   : in    std_logic;  -- system reset, low active
    CLK   : in    std_logic;  -- 24MHz for 4Mbps
    SCKX  : in    std_logic_vector(2 downto 0);  -- prescaler
    PLS   : in    std_logic;  -- motor step pulse input
    EDG   : in    std_logic;  -- encoder pulse edge
    MODE  : in    std_logic;  -- select motor step pulse input
    STEP  : inout std_logic;  -- motor step command
    SCI   : inout std_logic_vector(4 downto 0));
end component;
signal STEP  : std_logic;
signal SCIS  : std_logic_vector(4 downto 0);  -- SCI state

component TXC port(           -- transmit controler
    RES   : in    std_logic;  -- system reset, low active
    MCK   : in    std_logic;  -- CPU clock
    DIN   : in    std_logic_vector(7 downto 0);  -- data bus
    CS    : in    std_logic;  -- chip select
    WR    : in    std_logic;  -- write enable
    CLK   : in    std_logic;  -- 24MHz for 4Mbps
    SCKX  : in    std_logic_vector(2 downto 0);  -- prescaler
    SCI   : in    std_logic_vector(4 downto 0);
    STEP  : in    std_logic;  -- motor step command
    DIR   : in    std_logic;  -- direction of encoder
    MODE  : in    std_logic;  -- select motor step pulse input
    SD    : inout std_logic;  -- serial data inout
    OE    : inout std_logic;  -- output enable of SD
    FB    : out   std_logic); -- feedback of SD input
end component;
signal OE_fb : std_logic;  -- internal feedback of OE

component RXC port(           -- recieve controler
    RES   : in    std_logic;  -- system reset, low active
    DOUT  : out   std_logic_vector(3 downto 0);   -- data bus
    CS    : in    std_logic;  -- chip select
    RD    : in    std_logic;  -- read enable
    CLK   : in    std_logic;  -- 24MHz for 4Mbps
    SCKX  : in    std_logic_vector(2 downto 0);  -- prescaler
    SCI   : in    std_logic_vector(4 downto 0);
    STEP  : in    std_logic;  -- motor step command
    SD    : in    std_logic); -- serial data input
end component;

component ENC port(           -- rotary encoder interface
    RES   : in    std_logic;  -- system reset, low active
    CLK   : in    std_logic;  -- 24MHz for 4Mbps
    A,B   : in    std_logic;  -- rotary encoder pulse
    SCKX  : in    std_logic_vector(2 downto 0);  -- prescaler
    DIR   : out   std_logic;  -- direction of encoder
    EDG   : out   std_logic); -- encoder pulse edge
end component;

begin
    SCK   <= SCKX(2);
    OE    <= OE_fb;

U0: PRS port map (
--  RES, CLK, SCKX
    RES, CLK, SCKX);

U1: SCI port map (
--  RES, CLK, SCKX, PLS, EDG, MODE, STEP, SCI
    RES, CLK, SCKX, STP, EDG, MODE, STEP, SCIS);

U2: TXC port map (
--  RES, MCK, DIN, CS, WR, CLK, SCKX, SCI,  STEP, DIR, MODE, SD, OE,    FB
    RES, MCK, DIN, CS, WR, CLK, SCKX, SCIS, STEP, DIR, MODE, SD, OE_fb, FB);

U3: RXC port map (
--  RES, DOUT, CS, RD, CLK, SCKX, SCI,  STEP, SD
    RES, DOUT, CS, RD, CLK, SCKX, SCIS, STEP, SD);

U4: ENC port map (
--  RES, CLK, A, B, SCKX, DIR, EDG
    RES, CLK, A, B, SCKX, DIR, EDG);

end RTL;
