library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity TXC is port(
    RES  : in    std_logic;  -- system reset, low active
    MCK  : in    std_logic;  -- CPU clock
    DIN  : in    std_logic_vector(7 downto 0);  -- data bus
    CS   : in    std_logic;  -- chip select
    WR   : in    std_logic;  -- write enable
    CLK  : in    std_logic;  -- 24MHz for 4Mbps
    SCKX : in    std_logic_vector(2 downto 0);  -- prescaler
    SCI  : in    std_logic_vector(4 downto 0);
    STEP : in    std_logic;  -- motor step command
    DIR  : in    std_logic;  -- direction of encoder
    MODE : in    std_logic;  -- select motor step pulse input
    SD   : inout std_logic;  -- serial data
    OE   : inout std_logic;  -- output enable of SD
    FB   : out   std_logic); -- feedback of SD input
end TXC;

architecture RTL of TXC is
signal SCK3,SCK5,SCK6 : std_logic := '0';
signal TDR : std_logic_vector(7 downto 0) := "00000000";
signal TSD : std_logic := '0';  -- transmit serial data
signal SDO : std_logic := '0';  -- serial data out
signal DIR_M : std_logic := '0';  -- direction of motor rotation

begin
-- SCI clocks -------------------------------------------------
    SCK3 <= '1' when (SCKX = "011") else '0';  -- bit boundary
    SCK5 <= '1' when (SCKX = "101") else '0';
    SCK6 <= '1' when (SCKX = "110") else '0';  -- center of bit cell

-- transmit data register -------------------------------------
process (RES,MCK) begin
    if (RES = '0') then
            TDR <= "00000000";
    elsif (MCK'event and MCK = '0') then
        if (CS = '0') and (WR = '0') then
            TDR <= DIN;
        else
            TDR <= TDR;
        end if;
    end if;
end process;
    DIR_M <= DIR when (MODE = '1') else TDR(3);

-- TSD serializer ---------------------------------------------
process (RES,CLK) begin
    if (RES = '0') then
            TSD <= '0';  -- stop bit
    elsif (CLK'event and CLK = '1') then
        if (SCK3 = '1') then
            case SCI is
                when "10000" => TSD <= '1';  -- start bit
                when "10001" => TSD <= TDR(0);
                when "10010" => TSD <= TDR(1);
                when "10011" => TSD <= TDR(2);
                when "10100" => TSD <= DIR_M;
                when "10101" => TSD <= TDR(4);
                when "10110" => TSD <= TDR(5);
                when "10111" => TSD <= TDR(6);
                when "11000" => TSD <= TDR(7);
                when "11001" => TSD <= STEP;
                when others  => TSD <= '0';  -- stop bit, dummy bit
            end case;
        else
            TSD <= TSD;
        end if;
    end if;
end process;

-- FSK modulator ----------------------------------------------
process (RES,CLK) begin
    if (RES = '0') then
            SDO <= '0';
    elsif (CLK'event and CLK = '1') then
        if (SCI = "11111") and (SCK5 = '1') then
            SDO <= SD;            -- get initial level
        elsif (SCK6 = '1') then   -- center of bit cell
            SDO <= SDO xor TSD;   -- if TSD ='1' then invert
        elsif (SCK3 = '1') then   -- bit boundary
            SDO <= not SDO;
        else
            SDO <= SDO;
        end if;
    end if;
end process;

-- SD output enable -------------------------------------------
process (RES,CLK) begin
    if (RES = '0') then
            OE <= '1';  -- inactive
    elsif (CLK'event and CLK = '1') then
        if (SCI = "11111") and (SCK6 = '1') then
            OE <= '0';  -- return to transmit mode
        elsif (SCI >= "11011") then
            OE <= '1';  -- recieve mode
        else
            OE <= '0';  -- transmit mode
        end if;
    end if;
end process;
    SD <= SDO when (OE = '0') else 'Z';  -- serial data output
    FB <= SD  when (OE = '1') else 'Z';

end RTL;
